<?php

/**
 * @file
 */

namespace Drupal\tmgmt_spreadsheet\Plugin\tmgmt_file;

use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\tmgmt\Data;
use Drupal\tmgmt\Entity\Job;
use Drupal\tmgmt\JobInterface;
use Drupal\tmgmt_file\Format\FormatInterface;
use PhpOffice\PhpSpreadsheet\IOFactory;
use PhpOffice\PhpSpreadsheet\Spreadsheet;
use PhpOffice\PhpSpreadsheet\Style\Protection;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class TmgmtSpreadsheetFormatPluginBase
 *
 * @package Drupal\tmgmt_spreadsheet\Plugin
 */
abstract class TmgmtSpreadsheetFormatPluginBase implements FormatInterface, ContainerFactoryPluginInterface {
  use StringTranslationTrait;

  /**
   * @var string $format
   */
  protected $format;

  /**
   * @var \PhpOffice\PhpSpreadsheet\Spreadsheet $spreadsheet
   */
  protected $spreadsheet;

  /**
   * @var \Drupal\Core\File\FileSystemInterface $file_system;
   */
  protected $file_system;

  /**
   * @var \Drupal\Core\Messenger\MessengerInterface $messenger
   */
  protected $messenger;

  /**
   * @var \Drupal\tmgmt\Data $tmgmt_data
   */
  protected $tmgmt_data;

  /**
   * {@inheritdoc}
   */
  public function __construct($configuration,
                              $plugin_id,
                              $plugin_definition,
                              FileSystemInterface $file_system,
                              MessengerInterface $messenger,
                              Data $tmgmt_data) {
    $this->file_system = $file_system;
    $this->messenger = $messenger;
    $this->tmgmt_data = $tmgmt_data;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('file_system'),
      $container->get('messenger'),
      $container->get('tmgmt.data')
    );
  }

  /**
   * @param string $format
   *   Set PhpSpreadsheet-compatible format string.
   */
  protected function setFormat($format) {
    $this->format = $format;
  }

  /**
   * Get PhpSpreadsheet-compatible format string.
   *
   * @return string
   */
  protected function getFormat() {
    return $this->format;
  }

  /**
   * {@inheritdoc}
   */
  public function export(JobInterface $job, $conditions = []) {
    // create empty spreadsheet
    $this->spreadsheet = new Spreadsheet();
    $this->spreadsheet
      ->getProperties()
      ->setCreated(\Drupal::time()->getRequestTime())
      ->setCustomProperty('TmgmtJobId', $job->id(), 'i')
      ->setCustomProperty('TmgmtSourceLanguage', $job->getRemoteSourceLanguage(), 's')
      ->setCustomProperty('TmgmtTargetLanguage', $job->getRemoteTargetLanguage(), 's');
    $this->spreadsheet
      ->getActiveSheet()
      ->getProtection()
      ->setSheet(true);
    // add header
    $header = [
      'A' => 'Key',
      'B' => 'Parent label',
      'C' => 'Label',
      'D' => 'Text',
    ];
    foreach ($header as $col => $text) {
      $this->spreadsheet
        ->getActiveSheet()
        ->setCellValue($col . '1', $text);
    }
    $this->spreadsheet
      ->getActiveSheet()
      ->freezePane('C2');
    $row = 2;
    foreach ($job->getItems($conditions) as $item) {
      $data = $this->tmgmt_data->filterTranslatable($item->getData());
      foreach ($data as $key => $value) {
        // key
        $coordinate = 'A' . $row;
        $this->spreadsheet
          ->getActiveSheet()
          ->setCellValue($coordinate, $item->id() . '][' . $key);
        // parent label
        $coordinate = 'B' . $row;
        $parent_label = isset($value['#parent_label'][0]) ? $value['#parent_label'][0] : '';
        $this->spreadsheet
          ->getActiveSheet()
          ->calculateColumnWidths()
          ->setCellValue($coordinate, $parent_label);
        // label
        $coordinate = 'C' . $row;
        $label = isset($value['#label']) ? $value['#label'] : '';
        $this->spreadsheet
          ->getActiveSheet()
          ->calculateColumnWidths()
          ->setCellValue($coordinate, $label);
        // text
        $coordinate = 'D' . $row;
        $text = isset($value['#text']) ? $value['#text'] : '';
        $this->spreadsheet
          ->getActiveSheet()
          ->calculateColumnWidths()
          ->setCellValue($coordinate, $text)
          ->getStyle($coordinate)
          ->getProtection()
          ->setLocked(Protection::PROTECTION_UNPROTECTED);
        // auto size cells
        foreach (['B', 'C', 'D'] as $index) {
          $this->spreadsheet
            ->getActiveSheet()
            ->getColumnDimension($index)
            ->setAutoSize(true);
        }
        $row++;
      }
    }
    // hide key column and select active cell
    $this->spreadsheet
      ->getActiveSheet()
      ->setSelectedCell('B1')
      ->getColumnDimension('A')
      ->setVisible(FALSE);
    $objWriter = IOFactory::createWriter($this->spreadsheet, $this->getFormat());
    ob_start();
    $objWriter->save('php://output');
    return ob_get_clean();
  }

  /**
   * {@inheritdoc}
   */
  public function import($imported_file, $is_file = TRUE) {
    $result = [];
    $rows = $this->spreadsheet
      ->getActiveSheet()
      ->getRowIterator(2); // skip header row
    foreach ($rows as $index => $row) {
      $key = $this->spreadsheet
        ->getActiveSheet()
        ->getCell('A' . $index)
        ->getValue();
      $result[$key]['#text'] = $this->spreadsheet
        ->getActiveSheet()
        ->getCell('D' . $index)
        ->getValue();
    }
    return $this->tmgmt_data->unflatten($result);
  }

  /**
   * {@inheritdoc}
   */
  public function validateImport($imported_file, $is_file = TRUE) {
    $real_path = $this->file_system->realpath($imported_file);
    try {
      $file_type = IOFactory::identify($real_path);
    }
    catch (\Exception $e) {
      $this->messenger->addError($e->getMessage());
      return FALSE;
    }
    if ($file_type != $this->getFormat()) {
      $this->messenger->addError($this->t('Missing file format, please choose another plugin.'));
      return FALSE;
    }
    $reader = IOFactory::createReader($this->getFormat());
    $this->spreadsheet = $reader->load($real_path);
    // check tmgmt job
    $job_id = $this->spreadsheet
      ->getProperties()
      ->getCustomPropertyValue('TmgmtJobId');
    if (!$job = Job::load($job_id)) {
      $this->messenger->addError($this->t('Internal TMGMT error, unable load job.'));
      return FALSE;
    }
    // check languages
    $source_language = $this->spreadsheet
      ->getProperties()
      ->getCustomPropertyValue('TmgmtSourceLanguage');
    $target_language = $this->spreadsheet
      ->getProperties()
      ->getCustomPropertyValue('TmgmtTargetLanguage');
    if ($source_language != $job->getRemoteSourceLanguage() || $target_language != $job->getRemoteTargetLanguage()) {
      $this->messenger->addError($this->t('Internal TMGMT error, missing languages.'));
      return FALSE;
    }
    return $job;
  }

  /**
   * Get short class name.
   */
  public function getClassName() {
    return (new \ReflectionClass($this))->getShortName();
  }
}
